###########################################################################
#
# lucenebuildproc.pm -- perl wrapper for building index with Lucene
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package lucenebuildproc;

# This document processor outputs a document
# for lucene to process

# Use same basic XML structure setup by mgppbuilder/mgppbuildproc

use mgppbuildproc; 


sub BEGIN {
    @ISA = ('mgppbuildproc');
}


sub new {
    my $class = shift @_;
    my $self = new mgppbuildproc (@_);

    return bless $self, $class;
}

sub text {
    my $self = shift (@_);
    my ($doc_obj,$file) = @_;
    my $handle = $self->{'output_handle'};
    my $outhandle = $self->{'outhandle'};
    my $indexed_doc = 1;

    # only output this document if it is one to be indexed
    return if ($doc_obj->get_doc_type() ne "indexed_doc");

    # see if this document belongs to this subcollection
    foreach my $indexexp (@{$self->{'indexexparr'}}) {
	$indexed_doc = 0;
	my ($field, $exp, $options) = split /\//, $indexexp;
	if (defined ($field) && defined ($exp)) {
	    my ($bool) = $field =~ /^(.)/;
	    $field =~ s/^.// if $bool eq '!';
	    if ($field =~ /^filename$/i) {
		$field = $doc_obj->get_source_filename();
	    } else {
		$field = $doc_obj->get_metadata_element($doc_obj->get_top_section(), $field);
	    }
	    next unless defined $field;
	    if ($bool eq '!') {
		if ($options =~ /^i$/i) {
		    if ($field !~ /$exp/i) {$indexed_doc = 1; last;}
		} else {
		    if ($field !~ /$exp/) {$indexed_doc = 1; last;}
		}
	    } else {
		if ($options =~ /^i$/i) {
		    if ($field =~ /$exp/i) {$indexed_doc = 1; last;}
		} else {
		    if ($field =~ /$exp/) {$indexed_doc = 1; last;}
		}
	    }
	}
    }

    # this is another document
    $self->{'num_docs'} += 1;

    # get the parameters for the output
    # split on : just in case there is subcoll and lang stuff
    my ($fields) = split (/:/, $self->{'index'});

    my $doc_level = $mgppbuildproc::level_map{'document'};
    my $gs2ns = 'xmlns:gs2="http://www.greenstone.org/gs2"';

    my $levels = $self->{'levels'};
    my $ldoc_level = $levels->{'document'};
    my $lsec_level = $levels->{'section'};
    my $lpar_level = $levels->{'paragraph'};

    my $doc_idx_att = ($ldoc_level) ? "index=\"1\"" : "";
    my $sec_idx_att = ($lsec_level) ? "index=\"1\"" : "";
    my $par_idx_att = ($lpar_level) ? "index=\"1\"" : "";

    my $docid="";
    if ($ldoc_level) {
	my $doc_sec_num = $self->{'num_sections'}+1;	
	$docid = "gs2:id=\"$doc_sec_num\"";
    }

    my $documenttag = "<$doc_level $gs2ns file=\"$file\" $docid >\n";
    my $documentendtag = "\n</$doc_level>\n";
    
    my ($sectiontag) = "";    
    if ($lsec_level) {
	$sectiontag = $mgppbuildproc::level_map{'section'};
    }
    my ($paratag) = "";
    if ($self->{'levels'}->{'paragraph'}) {
	if ($self->{'strip_html'}) {
	    $paratag = "<". $mgppbuildproc::level_map{'paragraph'} . ">";
	} else {
	    print $outhandle "Paragraph level can not be used with no_strip_html!. Not indexing Paragraphs.\n";
	}
    }

    my $doc_section = 0; # just for this document

##    my $text = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>\n'; ## ****
    my $text = "";
    $text .= $documenttag;
   
    # get the text for this document
    my $section = $doc_obj->get_top_section();
    while (defined $section) {
	# update a few statistics
	$doc_section++;
	$self->{'num_sections'} += 1;
 
	if ($sectiontag ne "") {
	    my $secid = "gs2:id=\"".$self->{'num_sections'}."\"";	    
	    $text .= "\n<$sectiontag $secid >\n";
	}

	if ($indexed_doc) {
	    if ($self->{'indexing_text'}) {
		$text .= "$paratag"; # only add para tags for indexing
		# note that we assume that metadata will not be asked for for the compressed text, so we add para tags without checking for indexing_text
	    }
	    $self->{'num_bytes'} += $doc_obj->get_text_length ($section);
	    foreach my $field (split (/,/, $fields)) {
		# only deal with this field if it doesn't start with top or
		# this is the first section
		my $real_field = $field;
		if (!($real_field =~ s/^top//) || ($doc_section == 1)) {
		    my $new_text = ""; 
		    my $tmp_text = "";
		    if ($real_field eq "text") {
			if ($self->{'indexing_text'}) { #tag the text with <Text>...</Text>, add the <Paragraph> tags and strip out html if needed
			    $new_text .= "$paratag<TX index=\"1\">\n";
			    $tmp_text .= $doc_obj->get_text ($section);
			    $tmp_text = $self->preprocess_text($tmp_text, $self->{'strip_html'}, "</TX>$paratag<TX index=\"1\">");
			    
			    $new_text .= "$tmp_text</TX>\n";
			    #if (!defined $self->{'indexfields'}->{'TextOnly'}) {
				#$self->{'indexfields'}->{'TextOnly'} = 1;   
			    #}
			}
			else { # leave html stuff in, and dont add Paragraph tags - never retrieve paras at the moment
			    $new_text .= $doc_obj->get_text ($section) if $self->{'store_text'};
			}
		    } else { # metadata field
			if ($real_field eq "allfields") { #ignore
			}
			elsif ($real_field eq "metadata") { # insert all metadata 
			    #except gsdl stuff
			    my $shortname = "";
			    my $metadata = $doc_obj->get_all_metadata ($section);
			    foreach $pair (@$metadata) {
				my ($mfield, $mvalue) = (@$pair);
				# check fields here, maybe others dont want - change to use dontindex!!
				if ($mfield ne "Identifier" 
				    && $mfield !~ /^gsdl/ 
				    && $mfield ne "classifytype"
				    && $mfield ne "assocfilepath"
				    && defined $mvalue && $mvalue ne "") {
				    
				    if (defined $self->{'indexfieldmap'}->{$mfield}) {
					$shortname = $self->{'indexfieldmap'}->{$mfield};
				    }
				    else {
					$shortname = $self->create_shortname($mfield);
					$self->{'indexfieldmap'}->{$mfield} = $shortname;
					$self->{'indexfieldmap'}->{$shortname} = 1;
				    }	   
				    $new_text .= "$paratag<$shortname index=\"1\">$mvalue</$shortname>\n";
				    if (!defined $self->{'indexfields'}->{$mfield}) {
					$self->{'indexfields'}->{$mfield} = 1;
				    }				    
				}
			    }
			    
			}
			else { #individual metadata specified
			    my $shortname="";
			    #if (!defined $self->{'indexfields'}->{$real_field}) {
				#$self->{'indexfields'}->{$real_field} = 1;
			    #}
			    if (defined $self->{'indexfieldmap'}->{$real_field}) {
				$shortname = $self->{'indexfieldmap'}->{$real_field};
			    }
			    else {
				$shortname = $self->create_shortname($real_field);
				$self->{'indexfieldmap'}->{$real_field} = $shortname;
				$self->{'indexfieldmap'}->{$shortname} = 1;
			    }
			    foreach $item (@{$doc_obj->get_metadata ($section, $real_field)}) {
				$new_text .= "$paratag<$shortname index=\"1\">$item</$shortname>\n";
			    }
			}

		    }
		    
		    # filter the text
		    $self->filter_text ($field, $new_text);

		    $self->{'num_processed_bytes'} += length ($new_text);
		    $text .= "$new_text";
		}
	    }
	} # if (indexed_doc)
	
	$text .= "\n</$sectiontag>\n" if ($sectiontag ne "");

	$section = $doc_obj->get_next_section($section);
    } #while defined section
    print $handle "$text\n$documentendtag"; 
    
}

1;

